#include "hal.h"


// CONFIG2
#pragma config POSCMOD = NONE           // Primary Oscillator Select (Primary oscillator disabled)
#pragma config ALTI2C = OFF             // Alternate I2C pins (I2C mapped to SDA1/SCL1)
#pragma config IOL1WAY = ON             // IOLOCK Protection (Allow Only One Re-configuration)
#pragma config OSCIOFNC = OFF           // Primary Oscillator Output Function (OSC2 pin has clock out function)
#pragma config FCKSM = CSDCMD           // Clock Switching and Monitor (Clock switching and Fail-Safe Clock Monitor are disabled)
#pragma config FNOSC = FRCPLL           // Oscillator Mode (Internal Fast RC (FRC) w/ PLL)
#pragma config WDTWIN = WDTWIN25        // Watchdog Window Select (Watchdog Window is 25% of WDT period)
#pragma config IESO = OFF               // Internal External Switch Over Mode (Start-up device with user-selected oscillator source)

// CONFIG1
#pragma config WDTPOST = PS32768        // Watchdog Timer Postscaler (1:32,768)
#pragma config WDTPRE = PR128           // WDT Prescaler (Prescaler ratio of 1:128)
#pragma config PLLKEN = ON              // PLL Lock Enable (Clock switch to PLL source will wait until the PLL lock signal is valid.)
#pragma config WINDIS = OFF             // Watchdog Timer Window (Watchdog Timer in Non-Window mode)
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (Watchdog timer enabled/disabled by user software)
#pragma config ICS = PGD1               // Comm Channel Select (Communicate on PGEC1/PGED1)
#pragma config GWRP = OFF               // General Code Segment Write Protect (Writes to program memory are allowed)
#pragma config GCP = OFF                // General Segment Code Protection (General Segment Code protect is disabled)


char cmd_buffer[16];
int  cmd_buffer_idx = 0;

void hal_oscilator_init(void)
{
    // Configure Oscillator
    
    CLKDIVbits.FRCDIV  = 2;     // Oscilator postscaler set to 4
    CLKDIVbits.DOZEN   = 0;     // peripherial/cpu clock 1:1
    CLKDIVbits.DOZE    = 0;     // No clock reduction

}

void hal_ioport_init(void)
{
    // Inputs

    TRISAbits.TRISA4  = 1;  //DSW1
    TRISBbits.TRISB5  = 1;  //DSW2
    TRISBbits.TRISB10 = 1;  //DSW3
    TRISBbits.TRISB11 = 1;  //DSW4
    TRISBbits.TRISB12  = 1; //DSW5
    TRISBbits.TRISB13  = 1; //DSW8
    TRISBbits.TRISB14  = 1; //DSW7
    TRISBbits.TRISB15  = 1; //DSW6

    TRISAbits.TRISA2  = 1; //!ENABLE

    // Pullups on inputs

    CNPU1bits.CN0PUE = 1;  // DSW1
    CNPU2bits.CN27PUE = 1; // DSW2
    CNPU2bits.CN16PUE = 1; // DSW3
    CNPU1bits.CN15PUE = 1; // DSW4
    CNPU1bits.CN14PUE = 1; // DSW5
    CNPU1bits.CN13PUE = 1; // DSW8
    CNPU1bits.CN12PUE = 1; // DSW7
    CNPU1bits.CN11PUE = 1; // DSW6


    // RP3/RTB2-> U1RX
    TRISBbits.TRISB2  = 1; RPINR18bits.U1RXR = 0x02; //Input tied to RP2

     // Outputs

    // RP3-> U1TX
    TRISBbits.TRISB3  = 0; RPOR1bits.RP3R    = 0b00011;// RPn tied to UART1 Transmit
    
    TRISAbits.TRISA0   = 0;  // CPU_DIR
    TRISAbits.TRISA1   = 0;  // CPU_STEP
    TRISBbits.TRISB4   = 0;  // CPU_RESET
    TRISBbits.TRISB6   = 0;  // !CPU_CS

    TRISAbits.TRISA3   = 0;  // CPU_LED


    // Analog inputs disabled (digital)
    AD1PCFGL = 0xFFFF;


    // release L6474 from reset
    PORTBbits.RB4  = 1;

    // CPU_DIR/CPU_STEP low

    PORTAbits.RA0  = 0;
    PORTAbits.RA1  = 0;
    
    // CPU LED high
    PORTAbits.RA3  = 1;

}

void hal_uart_init()
{
    /*
    	 UART Configuration
    */
    U1MODE  = 0;

    U1MODEbits.STSEL = 0;	// 1-stop bit
    U1MODEbits.PDSEL = 0;	// No Parity, 8-data bits
    U1MODEbits.ABAUD = 0;	// Autobaud Disabled
    U1MODEbits.BRGH  = 0;       // Normal speed mode x16
    U1MODEbits.UEN   = 0;       // only tx/rx pins used

    U1BRG = ((FCY/4800)/16)-1 ;	// Baudrate

    /*
       Configure UART for Tx DMA transfers / Rx interrupt
    */
    U1STAbits.UTXISEL0 = 0;			// Interrupt after one Tx character is transmitted
    U1STAbits.UTXISEL1 = 0;
    U1STAbits.URXISEL  = 0;			// Interrupt after one RX character is received

    /*
        Enable UART Rx and Tx
    */
    U1MODEbits.UARTEN   = 1;		// Enable UART
    U1STAbits.UTXEN     = 1;		// Enable UART Tx

    IFS0bits.U1RXIF = 0;			// Clear the Recieve Interrupt Flag
    IEC0bits.U1RXIE = 1;			// Enable Recieve Interrupts

    IFS0bits.U1TXIF = 0;			// Clear the Transmit Interrupt Flag
}



void  hal_spi_init(void)
{
    SPI1STATbits.SPIEN = 0;

    SPI1CON1bits.DISSCK = 0;
    SPI1CON1bits.DISSDO = 0;
    SPI1CON1bits.MODE16 = 0;
    SPI1CON1bits.SMP    = 0;
    SPI1CON1bits.CKE    = 0;
    SPI1CON1bits.SSEN   = 0;
    SPI1CON1bits.CKP    = 1;
    SPI1CON1bits.MSTEN  = 1;

    SPI1CON1bits.SPRE   = 0; // 8:1
    SPI1CON1bits.PPRE   = 2; // 4:1

    SPI1CON2bits.FRMEN = 0;

    SPI1STATbits.SPIEN = 1;
}

uint8_t hal_spi_write_byte(uint8_t cc)
{
    uint8_t result;

    PORTBbits.RB6   = 0;  // !CPU_CS low

    SPI1BUF = cc;

    while(SPI1STATbits.SPIRBF == 0)
    {
    }

    result =  SPI1BUF;

    PORTBbits.RB6   = 1;  // !CPU_CS high

    return result;
}


 uint16_t hal_read_inputs(void)
{
    uint16_t dsw = 0;
    
#define DSW(n) ( 1<<(n-1))
    
    if(PORTAbits.RA4 == 0 )dsw |= DSW(1);
    if(PORTBbits.RB5 == 0 )dsw |= DSW(2);
    if(PORTBbits.RB10 == 0 )dsw |= DSW(3);
    if(PORTBbits.RB11 == 0 )dsw |= DSW(4);
    if(PORTBbits.RB12 == 0 )dsw |= DSW(5);
    if(PORTBbits.RB13 == 0 )dsw |= DSW(8);
    if(PORTBbits.RB14 == 0 )dsw |= DSW(7);
    if(PORTBbits.RB15 == 0 )dsw |= DSW(6);

    if(PORTAbits.RA2 != 0 )dsw |=  0x8000;
                
    return dsw;          
}


void hal_reset(void)
{
    RCONbits.SWDTEN=1;
    CORCONbits.IPL3 = 1;
    while(1)
    {
		;
    }
}


void hal_uart_puts (char *s)
{
    while(*s)
    {
        hal_uart_putc(*s);
        s++;
    } 
}


void hal_uart_putc (char c)
{
    while (U1STAbits.UTXBF ==1)
    {

    };
    U1TXREG = c;
}

void __attribute__ ((interrupt, no_auto_psv)) _U1RXInterrupt(void)
{
    uint16_t cc;

    while(U1STAbits.URXDA)
    {
        cc = U1RXREG;

        if(cmd_buffer_idx == 0)
        {
            if(cc =='$')
            {
                cmd_buffer[cmd_buffer_idx++] = cc;
            }
        }
        else
        {
            if(cmd_buffer_idx == sizeof(cmd_buffer))
            {
                cmd_buffer_idx = 0;
                continue;
            }

            if( (cc == '\r') || (cc == '\n'))
            {
              cmd_buffer[cmd_buffer_idx++] = 0;
              process_request(cmd_buffer);
              cmd_buffer_idx = 0;
            }
            else
            {
             cmd_buffer[cmd_buffer_idx++] = cc;
            }
        }    
    }

    IFS0bits.U1RXIF = 0;
}